class Assignment < ApplicationRecord
  belongs_to :user
  belongs_to :session

  validates :user_id, uniqueness: { scope: :session_id, message: "has already been assigned to this session" }

  validate :no_overlapping_assignments

  after_create_commit :notify_assignment_created
  after_destroy_commit :notify_assignment_destroyed

  scope :future, -> { joins(:session).where("sessions.starts_at" => Time.now..) }

  after_create_commit -> {
    Rails.logger.debug("Created assignment, broadcasting")
    broadcast_replace_to "sessions",
      target: session,
      partial: "sessions/session",
      locals: { session: session }
  }

  private

  def no_overlapping_assignments
    return if session.blank? || user.blank?

    overlapping_assignments = user.assignments
                                  .joins(:session)
                                  .where.not(id: id)
                                  .where(
                                    "sessions.starts_at < ? AND sessions.ends_at > ?",
                                    session.ends_at, session.starts_at
                                  )

    logger.debug(overlapping_assignments)
    return unless overlapping_assignments.exists?

    errors.add(:base, "This assignment overlaps with another assignment for this user.")
  end

  def notify_assignment_created
    ActiveSupport::Notifications.instrument("assignment.created", record: self)
  end

  def notify_assignment_destroyed
    ActiveSupport::Notifications.instrument("assignment.destroyed", record: self)
  end
end
